/******************************************************************************
*
* project name:    TICHESS
* project version: 1.00
* file name:       gui.c
* file version:    1.05
* initial date:    24/02/2000
* author:          thomas.nussbaumer@peem.com
* description:     contents most of the GUI stuff
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
*
* v1.00: 24/02/2000
*        extracted from tichess.c                           
*
* v1.01: 27/02/2000
*        UpdateMoveDisplay() fixed for rochade display (won't left garbage)
*
* v1.02: 28/02/2000
*        UpdateMoveDisplay() fixed for rochade display (won't left garbage)
*
* v1.03: 29/02/2000
*        new function HandleComputerMove() for more clearance
*
* v1.04: 01/03/2000
*        graphical artifacts removed (line height increased)
*        two real player mode added (key 'p')
*
* v1.05: 02/03/2000
*        ported to TI89    
*
******************************************************************************/

#include <nostub.h>
#include <all.h>
#include "tichess.h"


#if TI92_VERSION
//-----------------------------------------------------------------------------
// GRAPHICAL CONSTANTS FOR TI92plus
//-----------------------------------------------------------------------------
#define DISPLAY_TURN_X   126     // x position of turn display
#define DISPLAY_TURN_Y     1     // y position of turn display
#define DISPLAY_TURN_W   "white"
#define DISPLAY_TURN_B   "black"
#define DISPLAY_MOVE_X   168     // x position of move display
#define DISPLAY_MOVE_Y     1     // y position of move display
#define DISPLAY_NODE_X   215     // x position of node display
#define DISPLAY_NODE_Y     1     // y position of node display
#define COORD_POS_X      112
#define COORD_POS_Y      112
#define OFFSET_X           0     // x offset of board display
#define OFFSET_Y           0     // y offset of board display
#define SIZE_X            14     // x field size of board display
#define SIZE_Y            14     // y field size of board display
#define TXT_START_X      120     // x text start position
#define TXT_START_Y       12     // y text start position
#define TXT_HEIGHT         9     // height of a text line
#define LINES_PER_PAGE    11     // minimum lines per page
#define TXT_END_X        239     // x text end position
#define TXT_END_Y        119     // y text end position
#define MAX_MOVES_STORED 500     // maximum number of stored moves
#define COORD_OFFSET_X   (OFFSET_X+SIZE_X/3)
#define COORD_OFFSET_Y   (OFFSET_Y+SIZE_Y/3)
#define KEY_ESCAPE       264
#define KEY_LEFT         337
#define KEY_RIGHT        340
#define KEY_UP           338
#define KEY_DOWN         344
#define KEY_ENTER         13
#else
//-----------------------------------------------------------------------------
// GRAPHICAL CONSTANTS FOR TI89
//-----------------------------------------------------------------------------
#define DISPLAY_TURN_X    20     // x position of turn display
#define DISPLAY_TURN_Y    87     // y position of turn display
#define DISPLAY_TURN_W   "Turn: White"
#define DISPLAY_TURN_B   "Turn: Black"
#define DISPLAY_MOVE_X    93     // x position of move display
#define DISPLAY_MOVE_Y     1     // y position of move display
#define DISPLAY_NODE_X   136     // x position of node display
#define DISPLAY_NODE_Y     1     // y position of node display
#define COORD_POS_X       81
#define COORD_POS_Y       81
#define OFFSET_X           0     // x offset of board display
#define OFFSET_Y           0     // y offset of board display
#define SIZE_X            10     // x field size of board display
#define SIZE_Y            10     // y field size of board display
#define TXT_START_X       87     // x text start position
#define TXT_START_Y       10     // y text start position
#define TXT_HEIGHT         6     // height of a text line
#define LINES_PER_PAGE    11     // minimum lines per page

#define TXT_END_X        158     // x text end position
#define TXT_END_Y         92     // y text end position

#define MAX_MOVES_STORED 500     // maximum number of stored moves
#define COORD_OFFSET_X   (OFFSET_X+SIZE_X/3)
#define COORD_OFFSET_Y   (OFFSET_Y+SIZE_Y/3)
#define KEY_ESCAPE       264
#define KEY_LEFT         338
#define KEY_RIGHT        344
#define KEY_UP           337
#define KEY_DOWN         340
#define KEY_ENTER         13
#endif

//-----------------------------------------------------------------------------
// GLOBALS 
//-----------------------------------------------------------------------------

int      __x            =  TXT_START_X; // actual x position of text output
int      __y            =  TXT_START_Y; // actual y position of text output
SCR_RECT squarerect     = {};           // temporary rectangle
int      inverted       = FALSE;
int      select_x       =  0;           // selected square (x coordinate)
int      select_y       =  0;           // selected square (y coordinate)
int      idx_from       = -1;           // index into board of from-square
int      idx_to         = -1;           // index into board of to-square
int      select_promote = 0;            // used to store selected promotion fig
int      automatic_mode = FALSE;        // automatic mode setting
int      twoplayer_mode = FALSE;        // two player mode setting
char     jmp_buf[56]    = {};           // for longjump usage (exit or restart)
char     do_exit        = 0;            // for exit() handling
char*    move_store     = 0;            // the move storage buffer
int      stored         = 0;            // number of stored moves in move buffer
int      store_enabled  = 0;            // enable state of move buffer
char     drawn[BOARD_SIZE] = {};        // used to update board display


//-----------------------------------------------------------------------------
// HELPPAGE
//-----------------------------------------------------------------------------

#if TI92_VERSION
const char* helppage[] =
        { "\n========== HELP PAGE ==========\n"
          "L/S .... load/save state\n",
          "R   .... restart game\n",
          "C   .... change sides\n",
          "U/D .... level up/down\n",
          "A   .... toggle automatic play\n",
          "X   .... show moves so far\n",
          "B   .... go back to a move\n",
          "T   .... turn board by 180 degree\n",
          "P   .... toggle 2 player mode\n"
          "[thomas.nussbaumer@peem.com]\n",
          0};
#else
const char* helppage[] =
        { "\n==== HELP PAGE ====\n"
          "L/S - Load/Save\n",
          "R   - Restart\n",
          "C   - Change sides\n",
          "U/D - Level Up/Down\n",
          "A   - Automatic\n",
          "X   - Show moves\n",
          "B   - Back\n",
          "T   - Turn board\n",
          "P   - 2 player\n",
          "=================\n",
          0};
#endif

//-----------------------------------------------------------------------------
// declarations of the necessary "external" routines and variables
//-----------------------------------------------------------------------------

extern char* board;      // the board
extern int   act_color;  // which turn (white or black)
extern int   std_depth;  // standard depth
extern int   move_count; // number of moves so far
extern long  nr_nodes;

char*        Move2Str(move_t* pmove);
char*        Field2Notation(int fieldnumber,char* fieldnotation);
char         Figure2Char(char figure);
move_t*      ComputerMove(void);
int          UserMove(char* move);
void         InitEngine(int level);

#if DO_STATISTICS
void         ShowStatistics();
void         ToggleShowStats();
#endif                        


#if TI92_VERSION
//-----------------------------------------------------------------------------
// ICONS FOR TI92
//-----------------------------------------------------------------------------
ICON bicons[6] = {
 {0x0000,0x0000,0x1F80,0x1F80,0x1F80,0x1F80,0x1F80,0x1F80,0x0000,0x0000,0,0,0,0,0,0},
 {0xEF70,0xEF70,0xEF70,0xEF70,0xFFF0,0xFFF0,0xFFF0,0xFFF0,0xFFF0,0xFFF0,0,0,0,0,0,0},
 {0x0F00,0x0F00,0x0F00,0x0F00,0x0F00,0x0F00,0x1F80,0x1F80,0x3FC0,0x3FC0,0,0,0,0,0,0},
 {0x7FE0,0x7FE0,0x3FC0,0x1F80,0x0F00,0x0F00,0x1F80,0x3FC0,0x7FE0,0x7FE0,0,0,0,0,0,0},
 {0xC030,0x6060,0x3FC0,0x1F80,0x0F00,0x0F00,0x0F00,0x0F00,0x0F00,0x0F00,0,0,0,0,0,0},
 {0x0F00,0x0F00,0x0F00,0x7FE0,0x7FE0,0x7FE0,0x7FE0,0x0F00,0x0F00,0x0F00,0,0,0,0,0,0}};

ICON wicons[6] = {
 {0x0000,0x0000,0x1F80,0x1080,0x1080,0x1080,0x1080,0x1F80,0x0000,0x0000,0,0,0,0,0,0},
 {0xEF70,0xA950,0xA950,0xB9D0,0x8010,0x8010,0x8010,0x8010,0x8010,0xFFF0,0,0,0,0,0,0},
 {0x0F00,0x0900,0x0900,0x0900,0x0900,0x0900,0x1080,0x1080,0x2040,0x3FC0,0,0,0,0,0,0},
 {0x7FE0,0x4020,0x2040,0x1080,0x0900,0x0900,0x1080,0x2040,0x4020,0x7FE0,0,0,0,0,0,0},
 {0xC030,0x6060,0x3FC0,0x1080,0x0900,0x0900,0x0900,0x0900,0x0900,0x0F00,0,0,0,0,0,0},
 {0x0F00,0x0900,0x0900,0x79E0,0x4020,0x4020,0x79E0,0x0900,0x0900,0x0F00,0,0,0,0,0,0}};
#else 
//-----------------------------------------------------------------------------
// ICONS FOR TI89
//-----------------------------------------------------------------------------
ICON bicons[6] = {
{0,0,0x3f00,0x3f00,0x3f00,0x3f00,0x3f00,0x3f00,0,0,0,0,0,0,0,0},
{0,0x6d80,0x6d80,0x6d80,0x7f80,0x7f80,0x7f80,0x7f80,0x7f80,0,0,0,0,0,0,0},
{0,0xc00,0xc00,0x1e00,0x1e00,0x1e00,0x1e00,0x3f00,0x7f80,0,0,0,0,0,0,0},
{0,0x7f80,0x3f00,0x1e00,0x1e00,0x1e00,0x1e00,0x3f00,0x7f80,0,0,0,0,0,0,0},
{0,0x4c80,0x2d00,0x1e00,0x1e00,0x1e00,0x1e00,0x1e00,0x1e00,0,0,0,0,0,0,0},
{0,0x1e00,0x1e00,0x7f80,0x7f80,0x7f80,0x7f80,0x1e00,0x1e00,0,0,0,0,0,0,0}
};
ICON wicons[6] = {
{0,0,0x3f00,0x2100,0x2100,0x2100,0x2100,0x3f00,0,0,0,0,0,0,0,0},
{0,0x6d80,0x6d80,0x6d80,0x7f80,0x4080,0x4080,0x4080,0x7f80,0,0,0,0,0,0,0},
{0,0xc00,0xc00,0x1200,0x1200,0x1200,0x1200,0x2100,0x7f80,0,0,0,0,0,0,0},
{0,0x7f80,0x2100,0x1200,0x1200,0x1200,0x1200,0x2100,0x7f80,0,0,0,0,0,0,0},
{0,0x4c80,0x2d00,0x1200,0x1200,0x1200,0x1200,0x1200,0x1e00,0,0,0,0,0,0,0},
{0,0x1e00,0x1200,0x7380,0x4080,0x4080,0x7380,0x1200,0x1e00,0,0,0,0,0,0,0}
};
#endif


/*===========================================================================*/
/* printf output to terminal window                                          */
/*===========================================================================*/
#define printf(format,args...) \
{char printbuffer[200]; sprintf(printbuffer,format,##args);puts(printbuffer);}


/*===========================================================================*/
/* outputs one character to the terminal window                              */
/*===========================================================================*/
void putc(char c) {
    int i,j;
    char buffer[200];

    //--------------------------------------------------
    // clears line and positions cursor at the beginning
    //--------------------------------------------------
    if (c=='\r') {
        __x = TXT_START_X;
        i = (TXT_END_X-TXT_START_X)/FontCharWidth(' ');        
        for (j=0;j<i;j++) buffer[j] = ' ';
        buffer[j] = 0;
        for (i=0;i<2;i++) {
            SetPlane(i);
            DrawStr(__x,__y,buffer,A_REPLACE);
        }        
        return;
    }

    //------------------------------------------
    // wrap arround if necessary
    //------------------------------------------
    if (c=='\n'||__x+FontCharWidth(c)>TXT_END_X)
        __x=TXT_START_X,__y+=TXT_HEIGHT;

    //----------------------------------------------
    // don't scroll if it's not really necessary
    //----------------------------------------------
    if (c == '\n') return;

    if (__y+TXT_HEIGHT+2>=TXT_END_Y) {
        for (i=0;i<2;i++) {
            SetPlane(i);
            ScrRectScroll(&(SCR_RECT){TXT_START_X,TXT_START_Y,TXT_END_X,TXT_END_Y},
                          ScrRect,TXT_HEIGHT,A_REVERSE);
        }
        __y-=TXT_HEIGHT;
    }

    for (i=0;i<2;i++) {
        SetPlane(i);
        DrawChar(__x,__y,c,A_REPLACE);
    }
    __x+=FontCharWidth(c);

}


/*===========================================================================*/
/* outputs a string to terminal window                                       */
/*===========================================================================*/
void puts(const char *s) {
    while (*s != 0) putc(*s++);
}


/*===========================================================================*/
/* simple exit() implementation which does a long jump back into MainGUI()   */
/*===========================================================================*/
void LeavePrg(char* msg) {
    if (msg != 0) {
        puts(msg);
        puts("-------- PRESS KEY -------\n");
        ngetchx();
    }
    do_exit = 1;
    longjmp(jmp_buf,1);
}


/*===========================================================================*/
/* update the "TURN OF" display                                              */
/*===========================================================================*/
void UpdateTurnDisplay(int color) {
    int  i;
    for (i=0;i<2;i++) {
        SetPlane(i);
        if (color == WHITE) DrawStr(DISPLAY_TURN_X,DISPLAY_TURN_Y,DISPLAY_TURN_W,A_REPLACE);
        else                DrawStr(DISPLAY_TURN_X,DISPLAY_TURN_Y,DISPLAY_TURN_B,A_REPLACE);
    }
}


/*===========================================================================*/
/* update the "BEST MOVE SO FAR" display                                     */
/*===========================================================================*/
void UpdateMoveDisplay(move_t* pmove) {
    char s[20];
    int  i;
    sprintf(s,"%s     ",Move2Str(pmove));
    for (i=0;i<2;i++) {
        SetPlane(i);
        DrawStr(DISPLAY_MOVE_X,DISPLAY_MOVE_Y,s,A_REPLACE);
    }
}


/*===========================================================================*/
/* update the "NODES EXAMINED" display                                       */
/*===========================================================================*/
void UpdateNodeDisplay(long nodes) {
    char s[10];
    int  i;
    sprintf(s,"%06ld",nodes);
    for (i=0;i<2;i++) {
        SetPlane(i);
        DrawStr(DISPLAY_NODE_X,DISPLAY_NODE_Y,s,A_REPLACE);
    }
}

/*===========================================================================*/
/* draw coordinates                                                          */
/*===========================================================================*/
void DrawCoordinates() {
    int i,j;

    for (i=0;i<2;i++) {
        SetPlane(i);
        for (j=0;j<8;j++) {
            if (inverted == FALSE) {
                DrawChar(COORD_OFFSET_X+j*SIZE_X,COORD_POS_Y,'a'+j,A_REPLACE);
                DrawChar(COORD_POS_X,COORD_OFFSET_Y+j*SIZE_Y,'1'+7-j,A_REPLACE);
            }
            else {
                DrawChar(COORD_OFFSET_X+j*SIZE_X,COORD_POS_Y,'h'-j,A_REPLACE);
                DrawChar(COORD_POS_X,COORD_OFFSET_Y+j*SIZE_Y,'8'-7+j,A_REPLACE);
            }                
        }
    }
}


/*===========================================================================*/
/* set rectangle of field x,y                                                */
/*===========================================================================*/
void SetRect2Square(int x,int y) {
    squarerect.xy.x0 = OFFSET_X+SIZE_X*x;
    squarerect.xy.x1 = OFFSET_X+SIZE_X*(x+1)-1;
    squarerect.xy.y0 = OFFSET_Y+SIZE_Y*y;
    squarerect.xy.y1 = OFFSET_Y+SIZE_Y*(y+1)-1;
}


/*===========================================================================*/
/* maps cursor position to field value                                       */
/*===========================================================================*/
int Cursor2Idx(int x,int y) {
    if (inverted==TRUE) return 21 + y*10+(7-x);
    else                return 21 + (7-y)*10+x;
}


/*===========================================================================*/
/* draws complete board (if clear != 0 board will be cleared completely)     */
/*===========================================================================*/
void DrawCell(int x, int y,char s) {
    int i;

    ICON* picon;

    if (s == EMPTY) picon = 0;
    else if (s < 0) picon = &bicons[abs(s+1)];
    else            picon = &wicons[s-1];

    SetRect2Square(x,y);

    for (i=0;i<2;i++) {
        SetPlane(i);
        if (i==0 && ((y*8+x)%2)!=y%2) {
           ScrRectFill(&squarerect,ScrRect,A_REPLACE);
        }
        else {
           ScrRectFill(&squarerect,ScrRect,A_REVERSE);
        }
        if (s != EMPTY) {
#if TI92_VERSION
            DrawIcon(x*SIZE_X+OFFSET_X+1,y*SIZE_Y+OFFSET_Y+2,picon,A_REPLACE);
#else
            DrawIcon(x*SIZE_X+OFFSET_X,y*SIZE_Y+OFFSET_Y,picon,A_REPLACE);
#endif
        }
    }
}


/*===========================================================================*/
/* draws complete board                                                      */
/*===========================================================================*/
void DrawBoard() {
    int i,j,idx;
    char val;

    // update side to draw state display
    UpdateTurnDisplay(act_color);


    // check all squares for redraw ...
    for (i=0;i<8;i++) {
        for (j=0;j<8;j++) {
            idx = Cursor2Idx(i,j);
            val = board[idx];            
            if (drawn[idx] == val) {
                if (i!=select_x || j!=select_y) continue;
            }
            DrawCell(i,j,val);
            drawn[idx] = val;
        }
    }

    SetRect2Square(select_x,select_y);
    for (i=0;i<2;i++) {
        SetPlane(i);
        ScrRectFill(&squarerect,ScrRect,A_XOR);
    }
}


/*===========================================================================*/
/* initialize GUI (NOTE: GRAYMODE HAVE TO BE SET OUTSIDE !!!)                */
/*===========================================================================*/
void InitGUI() {
    int i,j;

    __x = TXT_START_X;
    __y = TXT_START_Y;

    select_x =  0;
    select_y =  7;
    idx_from = -1;
    idx_to   = -1;

    FontSetSys(F_4x6);

    for (i=0;i<2;i++) {
        SetPlane(i);
        ClrScr();
        DrawLine(TXT_START_X-1,7,TXT_END_X,7,A_REPLACE);
        DrawLine(TXT_START_X-2,0,TXT_START_X-2,TXT_END_Y+1,A_REPLACE);
        DrawLine(0,TXT_END_Y+1,TXT_END_X,TXT_END_Y+1,A_REPLACE);
#if TI92_VERSION
        DrawStr(TXT_START_X,1,"T:xxxxx        B:-------      N:000000",A_REPLACE);
        DrawStr(0,122,"TI-CHESS v1.00        [press H for Help]        (c) by TNUSSB (02/03/2000)",A_REPLACE);
#else 
        DrawStr(TXT_START_X,1,"B:-------    N:000000",A_REPLACE);
        //---------------------------------------------------------------------
        // Due to a really strange problem the bottom line MUST look like this.
        // (A part of the bottom line directly after the TI-CHESS string gets
        //  otherwise erased !!)
        //---------------------------------------------------------------------
        DrawStr(0,95,"TI-CHESS             [press H for Help]            v1.00",A_REPLACE);
#endif
    }

    for (i=0;i<BOARD_SIZE;i++) drawn[i] = -OUTSIDE;


    DrawCoordinates();
}


/*===========================================================================*/
/* handles a request dialog (returns TRUE if 'y' is pressed)                 */
/*===========================================================================*/
int RequestDialog(char* request) {
     int read;
     puts(request);
     do {
         read = toupper(ngetchx());
     }
     while (read != 'Y' && read != 'N');

     putc(read);
     putc('\n');
     if (read == 'Y') return TRUE;
     return FALSE;

}


/*===========================================================================*/
/* initialize the move storage                                               */
/*===========================================================================*/
void InitMoveStore() {
    stored        = 0;
    store_enabled = TRUE;
    move_store = (char*)malloc(MAX_MOVES_STORED*3);
}


/*===========================================================================*/
/* cleanup the move storage                                                  */
/*===========================================================================*/
void CleanupMoveStore() {
    if (move_store != 0) free(move_store);
    move_store = 0;
    stored     = 0;
}


/*===========================================================================*/
/* store a move in the move storage                                          */
/*===========================================================================*/
void StoreMove(char from,char to,char promote) {
    if (store_enabled == FALSE || move_store == 0 || stored >= MAX_MOVES_STORED) return;
    move_store[stored*3]   = from;
    move_store[stored*3+1] = to;
    move_store[stored*3+2] = promote;
    stored++;
}


/*===========================================================================*/
/* list all stored moves so far                                              */
/*===========================================================================*/
void ShowStoredMoves() {
    int  i = 0;
    char s1[10];
    char s2[10];
    char s3[10];
    char s4[10];

    if (move_store == 0 || stored == 0) return;

    for (i=0;i<stored;i+=2) {
        printf("[%03d] %s-%s",i,Field2Notation(move_store[i*3],s1),
                                Field2Notation(move_store[i*3+1],s2));
        if (i+1 < stored) {
            printf("  %s-%s",Field2Notation(move_store[(i+1)*3],s3),
                             Field2Notation(move_store[(i+1)*3+1],s4));
        }
        putc('\n');
        if (i % (LINES_PER_PAGE *2) == (LINES_PER_PAGE *2)-2) {
            puts("--- PRESS KEY ---");
            ngetchx();
            putc('\n');
        }
    }
}


/*===========================================================================*/
/* replays move from the move store                                          */
/*===========================================================================*/
void ReplayStoredMoves(int go_to_nr) {
    int i;
    automatic_mode = FALSE;
    InitGUI();
    InitEngine(std_depth);
    store_enabled = FALSE;
    for (i=0;i<go_to_nr;i++) {
        char s1[10];
        char s2[10];
        char s[20];
        sprintf(s,"%s%s%c",Field2Notation(move_store[i*3],s1),
                           Field2Notation(move_store[i*3+1],s2),
                           Figure2Char(move_store[i*3+2]));
        //printf("executing %s [%d]\n",s,i);
        UserMove(s);
    }
    store_enabled = TRUE;
}


/*===========================================================================*/
/* load game from String "ticsave"                                           */
/*===========================================================================*/
int HandleLoad() {
    SYM_ENTRY* symentry = SymFindPtr ($(ticsave), 0);
    char*      varptr;
    int        i;

    if (move_store == 0 || symentry==0) return FALSE;

    varptr = (char*)HeapDeref(symentry->handle);
    if (varptr == 0) return FALSE;

    stored = (((unsigned char)varptr[0])*256+((unsigned char)varptr[1])-3)/3;

    for (i=0;i<stored;i++) {
        move_store[i*3]   = varptr[3+i*3];
        move_store[i*3+1] = varptr[3+i*3+1];
        move_store[i*3+2] = varptr[3+i*3+2];
    }

    return TRUE;
}


/*===========================================================================*/
/* save game to String "ticsave"                                             */
/*                                                                           */
/* FORMAT: 3 chars specify a (half-)move                                     */
/*                                                                           */
/*         1. char = from field in internal notation                         */
/*         2. char = to field in internal notation                           */
/*         3. char = promoted figure (if promotion is done)                  */
/*                                                                           */
/*===========================================================================*/
int HandleSave() {
    SYM_ENTRY* symentry = DerefSym( SymAdd( $(ticsave) ) );
    char*      varptr;
    int i;

    if (symentry == 0) return FALSE;

    varptr = (char*)HeapDeref(symentry->handle = HeapAlloc(stored*3+5));
    if (varptr == 0) return FALSE;

    varptr[0] = (stored*3+3)/256;   // 2 bytes: length of the variable - 2
    varptr[1] = (stored*3+3)%256;
    varptr[2] = 0;   // zero marks a begining of the actual variable data

    for (i=0;i<stored;i++) {
        varptr[3+i*3]   = move_store[i*3];
        varptr[3+i*3+1] = move_store[i*3+1];
        varptr[3+i*3+2] = move_store[i*3+2];
    }

    varptr[3+stored*3] = 0;      // end-of-string marker
    varptr[4+stored*3] = 0x2D;   // the last byte is a type (0x2D = STR)

    return TRUE;
}


/*===========================================================================*/
/* handles a computer move                                                   */
/*===========================================================================*/
void HandleComputerMove() {
    move_t *executed = ComputerMove();
    UpdateNodeDisplay(nr_nodes);

#if DO_STATISTICS
    ShowStatistics();
#endif                        
    if (executed != 0) {
        StoreMove(executed->from,executed->to,executed->promoted_fig);
    }
    else {
        automatic_mode = FALSE;
    }
}                                


/*===========================================================================*/
/* nomen omen est                                                            */
/*===========================================================================*/
void MajorInputLoop(void) {
    int  read;
    int  tmpfig;
    int  i;
    char tmpnotation[10];

    do {
        DrawBoard();
        if (automatic_mode == FALSE || kbhit()) {
            read = tolower(ngetchx());
            automatic_mode = FALSE;
        }
        else {
            // automatic mode goes here ...
            HandleComputerMove();
            continue;
        }
        if (read == KEY_ESCAPE) break;  // ESCAPE PRESSED
        switch(read) {
            case KEY_LEFT: 
                if (select_x>0) {
                    // force redraw of previous
                    drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                    select_x--;
                }
                break;

            case KEY_RIGHT:
                if (select_x<7) {
                    // force redraw of previous
                    drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                    select_x++;
                }
                break;

            case KEY_DOWN:
                if (select_y<7) {
                    // force redraw of previous
                    drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                    select_y++;
                }
                break;

            case KEY_UP:
                if (select_y>0) {
                    // force redraw of previous
                    drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                    select_y--;
                }
                break;

            case KEY_ENTER: // ENTER
                if (idx_from == -1) {
                    i = Cursor2Idx(select_x,select_y);                        
                    tmpfig=board[i];
                    if (tmpfig == EMPTY ||
                       (act_color==WHITE && tmpfig<0) ||
                       (act_color==BLACK)&&(tmpfig>0)) break;

                    printf("[%03d] ",move_count);
                    if (act_color == BLACK) puts("..........  ");
                    printf("%c%s",Figure2Char(tmpfig),Field2Notation(i,tmpnotation));
                    idx_from = i;
                }
                else {
                    i = Cursor2Idx(select_x,select_y);
                    if (i == idx_from) {
                        putc('\r');
                        idx_from = -1;
                        break;
                    }
                    tmpfig=board[i];

                    if ((act_color==WHITE && tmpfig>0) ||
                       (act_color==BLACK)&&(tmpfig<0)) break;

                    printf("%c%s\n",(tmpfig==EMPTY)?'-':'x',Field2Notation(i,tmpnotation));
                    idx_to = i;

                    tmpfig = board[idx_from];
                    if ((act_color==WHITE && tmpfig==W_PAWN && idx_from >= A7) ||
                        (act_color==BLACK && tmpfig==B_PAWN && idx_from <= H2))
                    {
                        puts("??? PROMOTION TO Q/R/N/B ?  ");
                        do {
                            select_promote = toupper(ngetchx());
                        }
                        while (select_promote != KEY_ENTER  &&
                               select_promote != 'Q' &&
                               select_promote != 'R' &&
                               select_promote != 'N' &&
                               select_promote != 'B');

                        if (select_promote == KEY_ENTER) select_promote='Q';
                        putc(select_promote);
                        putc('\n');
                    }
                    else {
                       select_promote=0;
                    }

    
                    Field2Notation(idx_from,tmpnotation);
                    Field2Notation(idx_to,&tmpnotation[2]);
                    tmpnotation[4] = select_promote;
                    tmpnotation[5] = 0;
    
                    if (UserMove(tmpnotation)==FALSE) {
                        printf("ERROR: illegal move\n");
                    }
                    else {
                        StoreMove(idx_from, idx_to, select_promote);
                        if (twoplayer_mode == FALSE) {
                            DrawBoard();
                            HandleComputerMove();
                        }
                    }
                    idx_from = -1;
                }
                break;

            case 'h':
                i=0;
                while(helppage[i] != 0) puts(helppage[i++]);
                break;

            case 'r':
                if (RequestDialog("\nRestart game (Y/N)?")==TRUE) {
                   longjmp(jmp_buf,1);
                }
                break;

            case 'a':
                if (automatic_mode == FALSE) {
                   if (RequestDialog("\nStart automatic (Y/N)?")==TRUE) {
                      automatic_mode = TRUE;
                      twoplayer_mode = FALSE;
                   }
                }
                else {
                   automatic_mode = FALSE;
                   puts("automatic stopped.\n");
                }
                break;

            case 's':
                if (RequestDialog("\nSave game (Y/N)?")==TRUE) {
                    if (HandleSave()==TRUE) puts("successfully saved.\n");
                    else                    puts("saving failed.\n");
                }
                break;

            case 'l':
                if (RequestDialog("\nLoad game (Y/N)?")==TRUE) {
                    if (HandleLoad()==TRUE) {
                        ReplayStoredMoves(stored);
                        puts("successfully loaded.\n");
                    }
                    else {
                        puts("loading failed.\n");
                    }

                }
                break;

            case 'b':
                puts("\nEnter move number (3digits):\n");
                {
                    int count = 0;
                    int valid = FALSE;
                    int val   = 0;

                    do {
                        read = toupper(ngetchx());
                        if (valid == FALSE && read >= '0' && read <='9') {
                            putc(read);
                            switch(count) {
                                case 0: val += 100*(read-'0');
                                        break;
                                case 1: val += 10*(read-'0');
                                        break;
                                case 2: val += (read-'0');
                                        valid = TRUE;
                                        break;
                            }
                            count++;
                        }
                    }
                    while (read != KEY_ENTER && read != KEY_ESCAPE);
                    if (valid == FALSE) {
                        puts("\nERROR: NOT 3 digits\n");
                    }
                    else if (val >= stored) {
                        printf("\nERROR: %03d invalid\n",val);
                    }
                    else {
                        ReplayStoredMoves(val);
                        stored = val;
                    }
                }
                break;
            case 'c':
                if (RequestDialog("\nChange sides (Y/N)?")==TRUE) {
                    HandleComputerMove();
                }
                break;

            case 'u':
                std_depth++;
                if (std_depth > MAX_USERLEVEL) std_depth = MAX_USERLEVEL;
                printf("I'm now at level %d.\n",std_depth-1);
                break;

            case 'd':
                std_depth--;
                if (std_depth < MIN_USERLEVEL) std_depth = MIN_USERLEVEL;
                printf("I'm now at level %d.\n",std_depth-1);
                break;

            case 'x':
                puts("\n-- moves so far --\n");
                ShowStoredMoves();
                break;

            case 'p':
                if (twoplayer_mode == TRUE) {
                    if (RequestDialog("\nONE player (Y/N)?")==TRUE) {
                        twoplayer_mode = FALSE;
                    }
                }
                else {
                    if (RequestDialog("\nTWO player (Y/N)?")==TRUE) {
                        twoplayer_mode = TRUE;
                    }
                }                   
                break;

            case 't':
                if (inverted == TRUE) inverted = FALSE;
                else                  inverted = TRUE;
                
                for (i=0;i<BOARD_SIZE;i++) drawn[i] = -OUTSIDE;
                DrawCoordinates();
                DrawBoard();
                break;
#if DO_STATISTICS
            case 'z':
                ToggleShowStats();
                break;
#endif               
        }
    }
    while (1);
}



/*===========================================================================*/
/* main routine of the GUI                                                   */
/*===========================================================================*/
void MainGUI(int level) {    
    do_exit = 0;
    
    InitMoveStore();
    
    setjmp(jmp_buf);

    stored = 0;

    if (do_exit == 0) {
        InitGUI();
        automatic_mode = FALSE;
        twoplayer_mode = FALSE;
        InitEngine(level);
        printf("Hi. Try to beat me at level %d.\n",std_depth-1);
        MajorInputLoop();
    }

    CleanupMoveStore();
}


//##########################################################################
//###################### NO MORE FAKES BEYOND THIS LINE ####################
//##########################################################################
